from invana_bot.extractors.base import ExtractorBase


class AllLinksExtractor(ExtractorBase):
    pass


class SameDomainLinkExtractor(ExtractorBase):
    pass


class PaginationLinkExtractor(ExtractorBase):
    pass


class CustomLinkExtractor(ExtractorBase):
    pass
